@@ -2,12 +2,24 @@ import argparse
 import json
 import logging
 import os
+import re
 import sys
+import warnings
 from typing import Any, Dict, List, Optional
 
+from pystac import STACTypeError
+
 from .client import Client
+from .conformance import ConformanceClasses
 from .item_search import OPS
 from .version import __version__
+from .warnings import (
+    DoesNotConformTo,
+    FallbackToPystac,
+    MissingLink,
+    NoConformsTo,
+    PystacClientWarning,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -40,7 +52,7 @@ def search(
         return 0
 
     except Exception as e:
-        print(e)
+        print(f"ERROR: {e}")
         return 1
 
 
@@ -54,9 +66,78 @@ def collections(client: Client, save: Optional[str] = None) -> int:
         else:
             print(json.dumps(collections_dicts))
         return 0
-    except Exception as e:
-        print(e)
+    except STACTypeError as e:
+        print(f"ERROR: {e}")
+        print(
+            f"The client at {client.self_href} is OK, but one or more of the "
+            "collections is invalid."
+        )
         return 1
+    except Exception as e:
+        print(f"ERROR: {e}")
+        return 1
+
+
+def add_warnings_behavior(parser: argparse.ArgumentParser) -> None:
+    warnings_group = parser.add_argument_group("warnings behavior")
+    warnings_group.add_argument(
+        "--error",
+        nargs="*",
+        choices=[
+            "no-conforms-to",
+            "does-not-conform-to",
+            "missing-link",
+            "fallback-to-pystac",
+        ],
+        help="Error on all stac-client warnings or specific warnings.",
+    )
+    warnings_group.add_argument(
+        "--ignore",
+        nargs="*",
+        choices=[
+            "no-conforms-to",
+            "does-not-conform-to",
+            "missing-link",
+            "fallback-to-pystac",
+        ],
+        help="Ignore all stac-client warnings or specific warnings.",
+    )
+
+
+def set_warnings(error: Optional[List[str]], ignore: Optional[List[str]]) -> None:
+    # First set filters on the base class
+    if ignore is not None and len(ignore) == 0:
+        warnings.filterwarnings("ignore", category=PystacClientWarning)
+    if error is not None and len(error) == 0:
+        warnings.filterwarnings("ignore", category=PystacClientWarning)
+
+    # Then set filters on any specific classes
+    category_options = {
+        "no-conforms-to": NoConformsTo,
+        "does-not-conform-to": DoesNotConformTo,
+        "missing-link": MissingLink,
+        "fallback-to-pystac": FallbackToPystac,
+    }
+    if ignore is not None:
+        for w in ignore:
+            warnings.filterwarnings("ignore", category=category_options[w])
+    if error is not None:
+        for w in error:
+            warnings.filterwarnings("error", category=category_options[w])
+
+
+def set_conforms_to(
+    client: Client, clear: bool, remove: Optional[List[str]], add: Optional[List[str]]
+) -> None:
+    """Alters conforms_to settings on client in-place"""
+    if clear:
+        client.clear_conforms_to()
+    if remove is not None:
+        for conformance_class in remove:
+            client.remove_conforms_to(conformance_class)
+    if add is not None:
+        for conformance_class in add:
+            client.add_conforms_to(conformance_class)
 
 
 def parse_args(args: List[str]) -> Dict[str, Any]:
@@ -82,8 +163,19 @@ def parse_args(args: List[str]) -> Dict[str, Any]:
         default=None,
     )
     parent.add_argument(
-        "--ignore-conformance",
-        dest="ignore_conformance",
+        "--add-conforms-to",
+        choices=[c.name for c in ConformanceClasses],
+        nargs="*",
+        help="Specify conformance classes to add to client",
+    )
+    parent.add_argument(
+        "--remove-conforms-to",
+        choices=[c.name for c in ConformanceClasses],
+        nargs="*",
+        help="Specify conformance classes to remove from client",
+    )
+    parent.add_argument(
+        "--clear-conforms-to",
         default=False,
         action="store_true",
     )
@@ -97,6 +189,8 @@ def parse_args(args: List[str]) -> Dict[str, Any]:
         parents=[parent],
         formatter_class=dhf,
     )
+    add_warnings_behavior(parser)
+
     output_group = parser.add_argument_group("output options")
     output_group.add_argument(
         "--save", help="Filename to save collections to", default=None
@@ -160,6 +254,7 @@ def parse_args(args: List[str]) -> Dict[str, Any]:
         type=int,
     )
     search_group.add_argument("--method", help="GET or POST", default="POST")
+    add_warnings_behavior(parser)
 
     output_group = parser.add_argument_group("output options")
     output_group.add_argument(
@@ -191,10 +286,11 @@ def parse_args(args: List[str]) -> Dict[str, Any]:
     # if headers provided, parse it
     if "headers" in parsed_args:
         new_headers = {}
+        splitter = re.compile("^([^=]+)=(.+)$")
         for head in parsed_args["headers"]:
-            parts = head.split("=")
-            if len(parts) == 2:
-                new_headers[parts[0]] = parts[1]
+            match = splitter.match(head)
+            if match:
+                new_headers[match.group(1)] = match.group(2)
             else:
                 logger.warning(f"Unable to parse header {head}")
         parsed_args["headers"] = new_headers
@@ -230,12 +326,19 @@ def cli() -> int:
     try:
         url = args.pop("url")
         headers = args.pop("headers", {})
-        ignore_conformance = args.pop("ignore_conformance")
-        client = Client.open(
-            url, headers=headers, ignore_conformance=ignore_conformance
+
+        set_warnings(args.pop("error", None), args.pop("ignore", None))
+
+        client = Client.open(url, headers=headers)
+        set_conforms_to(
+            client,
+            args.pop("clear_conforms_to"),
+            args.pop("remove_conforms_to", None),
+            args.pop("add_conforms_to", None),
         )
+
     except Exception as e:
-        print(e)
+        print(f"ERROR: {e}", file=sys.stderr)
         return 1
 
     if cmd == "search":
@@ -244,7 +347,9 @@ def cli() -> int:
         return collections(client, **args)
     else:
         print(
-            f"Command '{cmd}' is not a valid command, must be 'search' or 'collections'"
+            f"Command '{cmd}' is not a valid command. "
+            "must be 'search' or 'collections'",
+            file=sys.stderr,
         )
         return 1
 
