@@ -1,3 +1,5 @@
+import re
+import warnings
 from functools import lru_cache
 from typing import (
     TYPE_CHECKING,
@@ -36,13 +38,15 @@ from pystac_client.item_search import (
     QueryLike,
     SortbyLike,
 )
-from pystac_client.stac_api_io import StacApiIO
+from pystac_client.mixins import QUERYABLES_ENDPOINT, QueryablesMixin
+from pystac_client.stac_api_io import StacApiIO, Timeout
+from pystac_client.warnings import DoesNotConformTo, FallbackToPystac, NoConformsTo
 
 if TYPE_CHECKING:
     from pystac.item import Item as Item_Type
 
 
-class Client(pystac.Catalog):
+class Client(pystac.Catalog, QueryablesMixin):
     """A Client for interacting with the root of a STAC Catalog or API
 
     Instances of the ``Client`` class inherit from :class:`pystac.Catalog`
@@ -93,10 +97,11 @@ class Client(pystac.Catalog):
         url: str,
         headers: Optional[Dict[str, str]] = None,
         parameters: Optional[Dict[str, Any]] = None,
-        ignore_conformance: bool = False,
+        ignore_conformance: Optional[bool] = None,
         modifier: Optional[Callable[[Modifiable], None]] = None,
         request_modifier: Optional[Callable[[Request], Union[Request, None]]] = None,
         stac_io: Optional[StacApiIO] = None,
+        timeout: Optional[Timeout] = None,
     ) -> "Client":
         """Opens a STAC Catalog or API
         This function will read the root catalog of a STAC Catalog or API
@@ -107,11 +112,16 @@ class Client(pystac.Catalog):
                 made to any part of this Catalog/API.
             parameters: Optional dictionary of query string parameters to
                 include in all requests.
-            ignore_conformance : Ignore any advertised Conformance Classes in this
-                Catalog/API. This means that
+            ignore_conformance (DEPRECATED) : Ignore any advertised Conformance Classes
+                in this Catalog/API. This means that
                 functions will skip checking conformance, and may throw an unknown
                 error if that feature is
                 not supported, rather than a :class:`NotImplementedError`.
+
+                .. deprecated:: 0.7.0
+                    Conformance can be altered rather than ignored using methods like
+                    :meth:`clear_conforms_to` and :meth:`add_conforms_to`
+
             modifier : A callable that modifies the children collection and items
                 returned by this Client. This can be useful for injecting
                 authentication parameters into child assets to access data
@@ -146,6 +156,9 @@ class Client(pystac.Catalog):
             stac_io: A `StacApiIO` object to use for I/O requests. Generally, leave
                 this to the default. However in cases where customized I/O processing
                 is required, a custom instance can be provided here.
+            timeout: Optional float or (float, float) tuple following the semantics
+              defined by `Requests
+              <https://requests.readthedocs.io/en/latest/api/#main-interface>`__.
 
         Return:
             catalog : A :class:`Client` instance for this Catalog/API
@@ -157,22 +170,21 @@ class Client(pystac.Catalog):
             modifier=modifier,
             request_modifier=request_modifier,
             stac_io=stac_io,
+            timeout=timeout,
         )
-        search_link = client.get_search_link()
-        # if there is a search link, but no conformsTo advertised, ignore
-        # conformance entirely
-        # NOTE: this behavior to be deprecated as implementations become conformant
-        if client._stac_io and (
-            ignore_conformance
-            or (
-                client
-                and "conformsTo" not in client.extra_fields.keys()
-                and search_link
-                and search_link.href
-                and len(search_link.href) > 0
+
+        if ignore_conformance is not None:
+            warnings.warn(
+                (
+                    "The `ignore_conformance` option is deprecated and will be "
+                    "removed in the next major release. Instead use `set_conforms_to` "
+                    "or `add_conforms_to` to control behavior."
+                ),
+                FutureWarning,
             )
-        ):
-            client._stac_io.set_conformance(None)
+
+        if not client.has_conforms_to():
+            warnings.warn(NoConformsTo())
 
         return client
 
@@ -185,6 +197,7 @@ class Client(pystac.Catalog):
         parameters: Optional[Dict[str, Any]] = None,
         modifier: Optional[Callable[[Modifiable], None]] = None,
         request_modifier: Optional[Callable[[Request], Union[Request, None]]] = None,
+        timeout: Optional[Timeout] = None,
     ) -> "Client":
         """Open a STAC Catalog/API
 
@@ -196,24 +209,97 @@ class Client(pystac.Catalog):
                 headers=headers,
                 parameters=parameters,
                 request_modifier=request_modifier,
+                timeout=timeout,
+            )
+        else:
+            stac_io.update(
+                headers=headers,
+                parameters=parameters,
+                request_modifier=request_modifier,
+                timeout=timeout,
             )
 
-        client: Client = super().from_file(href, stac_io)  # type: ignore
-
-        client._stac_io._conformance = client.extra_fields.get(  # type: ignore
-            "conformsTo", []
-        )
+        client: Client = super().from_file(href, stac_io)
         client.modifier = modifier
 
         return client
 
-    def _supports_collections(self) -> bool:
-        return self._conforms_to(ConformanceClasses.COLLECTIONS) or self._conforms_to(
-            ConformanceClasses.FEATURES
+    def has_conforms_to(self) -> bool:
+        """Whether server contains list of ``"conformsTo"`` URIs"""
+        return "conformsTo" in self.extra_fields
+
+    def get_conforms_to(self) -> List[str]:
+        """List of ``"conformsTo"`` URIs
+
+        Return:
+            List[str]: List of  URIs that the server conforms to
+        """
+        return cast(List[str], self.extra_fields.get("conformsTo", []).copy())
+
+    def set_conforms_to(self, conformance_uris: List[str]) -> None:
+        """Set list of ``"conformsTo"`` URIs
+
+        Args:
+            conformance_uris : URIs indicating what the server conforms to
+        """
+        self.extra_fields["conformsTo"] = conformance_uris
+
+    def clear_conforms_to(self) -> None:
+        """Clear list of ``"conformsTo"`` urls
+
+        Removes the entire list, so :py:meth:`has_conforms_to` will
+        return False after using this method.
+        """
+        self.extra_fields.pop("conformsTo", None)
+
+    def add_conforms_to(self, name: str) -> None:
+        """Add ``"conformsTo"`` by name.
+
+        Args:
+            name : name of :py:class:`ConformanceClasses` keys to add.
+        """
+        conformance_class = ConformanceClasses.get_by_name(name)
+
+        if not self.conforms_to(conformance_class):
+            self.set_conforms_to([*self.get_conforms_to(), conformance_class.valid_uri])
+
+    def remove_conforms_to(self, name: str) -> None:
+        """Remove ``"conformsTo"`` by name.
+
+        Args:
+            name : name of :py:class:`ConformanceClasses` keys to remove.
+        """
+        conformance_class = ConformanceClasses.get_by_name(name)
+
+        self.set_conforms_to(
+            [
+                uri
+                for uri in self.get_conforms_to()
+                if not re.match(conformance_class.pattern, uri)
+            ]
         )
 
-    def _conforms_to(self, conformance_class: ConformanceClasses) -> bool:
-        return self._stac_io.conforms_to(conformance_class)  # type: ignore
+    def conforms_to(self, conformance_class: Union[ConformanceClasses, str]) -> bool:
+        """Checks whether the API conforms to the given standard.
+
+        This method only checks
+        against the ``"conformsTo"`` property from the API landing page and does not
+        make any additional calls to a ``/conformance`` endpoint even if the API
+        provides such an endpoint.
+
+        Args:
+            name : name of :py:class:`ConformanceClasses` keys to check
+                conformance against.
+
+        Return:
+            bool: Indicates if the API conforms to the given spec or URI.
+        """
+        if isinstance(conformance_class, str):
+            conformance_class = ConformanceClasses.get_by_name(conformance_class)
+
+        return any(
+            re.match(conformance_class.pattern, uri) for uri in self.get_conforms_to()
+        )
 
     @classmethod
     def from_dict(
@@ -235,40 +321,93 @@ class Client(pystac.Catalog):
                 f"Could not open Client (href={href}), "
                 f"expected type=Catalog, found type={d.get('type', None)}"
             )
-        # cast require for mypy to believe that we have a Client, rather than
-        # the super type.
-        # https://github.com/stac-utils/pystac/issues/862
-        result = cast(Client, result)
 
         result.modifier = modifier
         return result
 
+    def _supports_collections(self) -> bool:
+        return self.conforms_to(ConformanceClasses.COLLECTIONS) or self.conforms_to(
+            ConformanceClasses.FEATURES
+        )
+
+    def _warn_about_fallback(self, *args: str) -> None:
+        if self.has_conforms_to():
+            warnings.warn(DoesNotConformTo(*args), stacklevel=2)
+        warnings.warn(FallbackToPystac(), stacklevel=2)
+
+    def get_merged_queryables(self, collections: List[str]) -> Dict[str, Any]:
+        """Return the set of queryables in common to the specified collections.
+
+        Queryables from multiple collections are unioned together, except in the case
+        when the same queryable key has a different definition, in which case that key
+        is dropped.
+
+        Output is a dictionary that can be used in ``jsonshema.validate``
+
+        Args:
+            collections List[str]: The IDs of the collections to inspect.
+
+        Return:
+            Dict[str, Any]: Dictionary containing queryable fields
+        """
+        if not collections:
+            raise ValueError("cannot get_merged_queryables from empty Iterable")
+
+        if not self.conforms_to(ConformanceClasses.FILTER):
+            raise DoesNotConformTo(ConformanceClasses.FILTER.name)
+        response = self.get_queryables_from(
+            self._get_collection_queryables_href(collections[0])
+        )
+        response.pop("$id")
+        addl_props = response.get("additionalProperties", False)
+        for collection in collections[1:]:
+            resp = self.get_queryables_from(
+                self._get_collection_queryables_href(collection)
+            )
+
+            # additionalProperties is false if any collection doesn't support additional
+            # properties
+            addl_props &= resp.get("additionalProperties", False)
+
+            # drop queryables if their keys match, but the descriptions differ
+            for k in set(resp["properties"]).intersection(response["properties"]):
+                if resp["properties"][k] != response["properties"][k]:
+                    resp["properties"].pop(k)
+                    response["properties"].pop(k)
+            response["properties"].update(resp["properties"])
+        return response
+
     @lru_cache()
-    def get_collection(self, collection_id: str) -> Optional[Collection]:
+    def get_collection(self, collection_id: str) -> Union[Collection, CollectionClient]:
         """Get a single collection from this Catalog/API
 
         Args:
             collection_id: The Collection ID to get
 
         Returns:
-            CollectionClient: A STAC Collection
+            Union[Collection, CollectionClient]: A STAC Collection
+
+        Raises:
+            NotFoundError if collection_id does not exist.
         """
-        if self._supports_collections() and self._stac_io:
-            url = self._get_collections_href(collection_id)
+        collection: Union[Collection, CollectionClient]
+
+        if self._supports_collections():
+            assert self._stac_io is not None
+
+            url = self._collections_href(collection_id)
             collection = CollectionClient.from_dict(
                 self._stac_io.read_json(url),
                 root=self,
                 modifier=self.modifier,
             )
             call_modifier(self.modifier, collection)
-            return collection
         else:
-            for col in self.get_collections():
-                if col.id == collection_id:
-                    call_modifier(self.modifier, col)
-                    return col
-
-        return None
+            self._warn_about_fallback("COLLECTIONS", "FEATURES")
+            for collection in super().get_collections():
+                if collection.id == collection_id:
+                    call_modifier(self.modifier, collection)
+        return collection
 
     def get_collections(self) -> Iterator[Collection]:
         """Get Collections in this Catalog
@@ -277,12 +416,14 @@ class Client(pystac.Catalog):
             otherwise fall back to Catalog behavior of following child links
 
         Return:
-            Iterator[Collection]: Iterator over Collections in Catalog/API
+            Iterator[Union[Collection, CollectionClient]]: Collections in Catalog/API
         """
         collection: Union[Collection, CollectionClient]
 
-        if self._supports_collections() and self._stac_io:
-            url = self._get_collections_href()
+        if self._supports_collections():
+            assert self._stac_io is not None
+
+            url = self._collections_href()
             for page in self._stac_io.get_pages(url):
                 if "collections" not in page:
                     raise APIError("Invalid response from /collections")
@@ -293,6 +434,7 @@ class Client(pystac.Catalog):
                     call_modifier(self.modifier, collection)
                     yield collection
         else:
+            self._warn_about_fallback("COLLECTIONS", "FEATURES")
             for collection in super().get_collections():
                 call_modifier(self.modifier, collection)
                 yield collection
@@ -304,10 +446,11 @@ class Client(pystac.Catalog):
             Iterator[Item]:: Iterator of items whose parent is this
                 catalog.
         """
-        if self._conforms_to(ConformanceClasses.ITEM_SEARCH):
+        if self.conforms_to(ConformanceClasses.ITEM_SEARCH):
             search = self.search()
             yield from search.items()
         else:
+            self._warn_about_fallback("ITEM_SEARCH")
             for item in super().get_items():
                 call_modifier(self.modifier, item)
                 yield item
@@ -321,13 +464,7 @@ class Client(pystac.Catalog):
                 catalogs or collections connected to this catalog through
                 child links.
         """
-        if self._conforms_to(ConformanceClasses.ITEM_SEARCH):
-            # these are already modified
-            yield from self.get_items()
-        else:
-            for item in super().get_items():
-                call_modifier(self.modifier, item)
-                yield item
+        yield from self.get_items()
 
     def search(
         self,
@@ -446,29 +583,16 @@ class Client(pystac.Catalog):
                 or does not have a link with
                 a ``"rel"`` type of ``"search"``.
         """
-        if not self._conforms_to(ConformanceClasses.ITEM_SEARCH):
-            raise NotImplementedError(
-                "This catalog does not support search because it "
-                f'does not conform to "{ConformanceClasses.ITEM_SEARCH}"'
-            )
-        search_link = self.get_search_link()
-        if search_link:
-            if isinstance(search_link.target, str):
-                search_href = search_link.target
-            else:
-                raise NotImplementedError(
-                    "Link with rel=search was an object rather than a URI"
-                )
-        else:
-            raise NotImplementedError(
-                "No link with rel=search could be found in this catalog"
+
+        if not self.conforms_to(ConformanceClasses.ITEM_SEARCH):
+            raise DoesNotConformTo(
+                "ITEM_SEARCH", "There is not fallback option available for search."
             )
 
         return ItemSearch(
-            url=search_href,
+            url=self._search_href(),
             method=method,
             max_items=max_items,
-            stac_io=self._stac_io,
             client=self,
             limit=limit,
             ids=ids,
@@ -505,35 +629,20 @@ class Client(pystac.Catalog):
             None,
         )
 
-    def _get_collections_href(self, collection_id: Optional[str] = None) -> str:
-        self_href = self.get_self_href()
-        if self_href is None:
-            data_link = self.get_single_link("data")
-            if data_link is None:
-                raise ValueError(
-                    "cannot build a collections href without a self href or a data link"
-                )
-            else:
-                collections_href = data_link.href
-        else:
-            collections_href = f"{self_href.rstrip('/')}/collections"
+    def _search_href(self) -> str:
+        search_link = self.get_search_link()
+        href = self._get_href("search", search_link, "search")
+        return href
 
-        if not pystac.utils.is_absolute_href(collections_href):
-            collections_href = self._make_absolute_href(collections_href)
+    def _collections_href(self, collection_id: Optional[str] = None) -> str:
+        data_link = self.get_single_link("data")
+        href = self._get_href("data", data_link, "collections")
+        if collection_id is not None:
+            return f"{href.rstrip('/')}/{collection_id}"
+        return href
 
-        if collection_id is None:
-            return collections_href
-        else:
-            return f"{collections_href.rstrip('/')}/{collection_id}"
-
-    def _make_absolute_href(self, href: str) -> str:
-        self_link = self.get_single_link("self")
-        if self_link is None:
-            raise ValueError("cannot build an absolute href without a self link")
-        elif not pystac.utils.is_absolute_href(self_link.href):
-            raise ValueError(
-                "cannot build an absolute href from "
-                f"a relative self link: {self_link.href}"
-            )
-        else:
-            return pystac.utils.make_absolute_href(href, self_link.href)
+    def _get_collection_queryables_href(
+        self, collection_id: Optional[str] = None
+    ) -> str:
+        href = self._collections_href(collection_id)
+        return f"{href.rstrip('/')}/{QUERYABLES_ENDPOINT}"
